@@ -1,5 +1,7 @@
 use core::cmp::Ordering;
 
+use im_ternary_tree::TernaryTreeList;
+
 use crate::primes::{Calcit, CalcitErr, CalcitItems, CrListWrap};
 use crate::util::number::f64_to_usize;
 
@@ -7,8 +9,6 @@ use crate::builtins;
 use crate::call_stack::CallStackList;
 use crate::runner;
 
-use im_ternary_tree::TernaryTreeList;
-
 pub fn new_list(xs: &CalcitItems) -> Result<Calcit, CalcitErr> {
   Ok(Calcit::List(xs.to_owned()))
 }
@@ -66,7 +66,7 @@ pub fn append(xs: &CalcitItems) -> Result<Calcit, CalcitErr> {
     return CalcitErr::err_str(format!("append expected 2 arguments, got: {}", CrListWrap(xs.to_owned())));
   }
   match &xs[0] {
-    Calcit::List(ys) => Ok(Calcit::List(ys.push(xs[1].to_owned()))),
+    Calcit::List(ys) => Ok(Calcit::List(ys.push_right(xs[1].to_owned()))),
     a => CalcitErr::err_str(format!("append expected a list: {}", a)),
   }
 }
@@ -88,7 +88,7 @@ pub fn rest(xs: &CalcitItems) -> Result<Calcit, CalcitErr> {
       if ys.is_empty() {
         Ok(Calcit::Nil)
       } else {
-        Ok(Calcit::List(ys.rest()?))
+        Ok(Calcit::List(ys.drop_left()))
       }
     }
     a => CalcitErr::err_str(format!("list:rest expected a list, got: {}", a)),
@@ -105,9 +105,7 @@ pub fn butlast(xs: &CalcitItems) -> Result<Calcit, CalcitErr> {
       if ys.is_empty() {
         Ok(Calcit::Nil)
       } else {
-        let mut zs = ys.to_owned();
-        zs = zs.butlast()?;
-        Ok(Calcit::List(zs))
+        Ok(Calcit::List(ys.butlast()?))
       }
     }
     a => CalcitErr::err_str(format!("butlast expected a list, got: {}", a)),
@@ -119,7 +117,7 @@ pub fn concat(xs: &CalcitItems) -> Result<Calcit, CalcitErr> {
   for x in xs {
     if let Calcit::List(zs) = x {
       for z in zs {
-        ys = ys.push(z.to_owned());
+        ys = ys.push_right(z.to_owned());
       }
     } else {
       return CalcitErr::err_str(format!("concat expects list arguments, got: {}", x));
@@ -156,12 +154,12 @@ pub fn range(xs: &CalcitItems) -> Result<Calcit, CalcitErr> {
   let mut i = base;
   if step > 0.0 {
     while i < bound {
-      ys = ys.push(Calcit::Number(i));
+      ys = ys.push_right(Calcit::Number(i));
       i += step;
     }
   } else {
     while i > bound {
-      ys = ys.push(Calcit::Number(i));
+      ys = ys.push_right(Calcit::Number(i));
       i += step;
     }
   }
@@ -194,7 +192,7 @@ pub fn foldl(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Calcit, Cal
       ) => {
         for x in xs {
           let values = TernaryTreeList::from(&[ret, x.to_owned()]);
-          ret = runner::run_fn(&values, scope, args.to_owned(), body, def_ns.to_owned(), call_stack)?;
+          ret = runner::run_fn(&values, scope, args, body, def_ns.to_owned(), call_stack)?;
         }
         Ok(ret)
       }
@@ -214,7 +212,7 @@ pub fn foldl(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Calcit, Cal
       ) => {
         for x in xs {
           let values = TernaryTreeList::from(&[ret, x.to_owned()]);
-          ret = runner::run_fn(&values, scope, args.to_owned(), body, def_ns.to_owned(), call_stack)?;
+          ret = runner::run_fn(&values, scope, args, body, def_ns.to_owned(), call_stack)?;
         }
         Ok(ret)
       }
@@ -234,7 +232,7 @@ pub fn foldl(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Calcit, Cal
       ) => {
         for (k, x) in xs {
           let values = TernaryTreeList::from(&[ret, Calcit::List(TernaryTreeList::from(&[k.to_owned(), x.to_owned()]))]);
-          ret = runner::run_fn(&values, scope, args.to_owned(), body, def_ns.to_owned(), call_stack)?;
+          ret = runner::run_fn(&values, scope, args, body, def_ns.to_owned(), call_stack)?;
         }
         Ok(ret)
       }
@@ -280,7 +278,7 @@ pub fn foldl_shortcut(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Ca
         let mut state = acc.to_owned();
         for x in xs {
           let values = TernaryTreeList::from(&[state, x.to_owned()]);
-          let pair = runner::run_fn(&values, scope, args.to_owned(), body, def_ns.to_owned(), call_stack)?;
+          let pair = runner::run_fn(&values, scope, args, body, def_ns.to_owned(), call_stack)?;
           match pair {
             Calcit::Tuple(x0, x1) => match &*x0 {
               Calcit::Bool(b) => {
@@ -317,7 +315,7 @@ pub fn foldl_shortcut(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Ca
         let mut state = acc.to_owned();
         for x in xs {
           let values = TernaryTreeList::from(&[state, x.to_owned()]);
-          let pair = runner::run_fn(&values, scope, args.to_owned(), body, def_ns.to_owned(), call_stack)?;
+          let pair = runner::run_fn(&values, scope, args, body, def_ns.to_owned(), call_stack)?;
           match pair {
             Calcit::Tuple(x0, x1) => match &*x0 {
               Calcit::Bool(b) => {
@@ -354,7 +352,7 @@ pub fn foldl_shortcut(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Ca
         let mut state = acc.to_owned();
         for (k, x) in xs {
           let values = TernaryTreeList::from(&[state, Calcit::List(TernaryTreeList::from(&[k.to_owned(), x.to_owned()]))]);
-          let pair = runner::run_fn(&values, scope, args.to_owned(), body, def_ns.to_owned(), call_stack)?;
+          let pair = runner::run_fn(&values, scope, args, body, def_ns.to_owned(), call_stack)?;
           match pair {
             Calcit::Tuple(x0, x1) => match &*x0 {
               Calcit::Bool(b) => {
@@ -414,7 +412,7 @@ pub fn foldr_shortcut(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Ca
         for i in 0..size {
           let x = xs[size - 1 - i].to_owned();
           let values = TernaryTreeList::from(&[state, x]);
-          let pair = runner::run_fn(&values, scope, args.to_owned(), body, def_ns.to_owned(), call_stack)?;
+          let pair = runner::run_fn(&values, scope, args, body, def_ns.to_owned(), call_stack)?;
           match pair {
             Calcit::Tuple(x0, x1) => match &*x0 {
               Calcit::Bool(b) => {
@@ -467,8 +465,8 @@ pub fn sort(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Calcit, Calc
       ) => {
         let mut xs2: Vec<&Calcit> = xs.into_iter().collect::<Vec<&Calcit>>();
         xs2.sort_by(|a, b| -> Ordering {
-          let values = TernaryTreeList::from(&[a.to_owned().to_owned(), b.to_owned().to_owned()]);
-          let v = runner::run_fn(&values, scope, args.to_owned(), body, def_ns.to_owned(), call_stack);
+          let values = TernaryTreeList::from(&[(*a).to_owned(), (*b).to_owned()]);
+          let v = runner::run_fn(&values, scope, args, body, def_ns.to_owned(), call_stack);
           match v {
             Ok(Calcit::Number(x)) if x < 0.0 => Ordering::Less,
             Ok(Calcit::Number(x)) if x == 0.0 => Ordering::Equal,
@@ -486,14 +484,14 @@ pub fn sort(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Calcit, Calc
         let mut ys: TernaryTreeList<Calcit> = TernaryTreeList::Empty;
         for x in xs2.iter() {
           // TODO ??
-          ys = ys.push(x.to_owned().to_owned())
+          ys = ys.push_right((*x).to_owned())
         }
         Ok(Calcit::List(ys))
       }
       (Calcit::List(xs), Calcit::Proc(proc)) => {
         let mut xs2: Vec<&Calcit> = xs.into_iter().collect::<Vec<&Calcit>>();
         xs2.sort_by(|a, b| -> Ordering {
-          let values = TernaryTreeList::from(&[a.to_owned().to_owned(), b.to_owned().to_owned()]);
+          let values = TernaryTreeList::from(&[(*a).to_owned(), (*b).to_owned()]);
           let v = builtins::handle_proc(proc, &values, call_stack);
           match v {
             Ok(Calcit::Number(x)) if x < 0.0 => Ordering::Less,
@@ -512,7 +510,7 @@ pub fn sort(xs: &CalcitItems, call_stack: &CallStackList) -> Result<Calcit, Calc
         let mut ys: TernaryTreeList<Calcit> = TernaryTreeList::Empty;
         for x in xs2.iter() {
           // TODO ??
-          ys = ys.push(x.to_owned().to_owned())
+          ys = ys.push_right((*x).to_owned())
         }
         Ok(Calcit::List(ys))
       }
@@ -667,7 +665,7 @@ pub fn distinct(xs: &CalcitItems) -> Result<Calcit, CalcitErr> {
       let mut zs = TernaryTreeList::Empty;
       for y in ys {
         if zs.index_of(y).is_none() {
-          zs = zs.push(y.to_owned());
+          zs = zs.push_right(y.to_owned());
         }
       }
       Ok(Calcit::List(zs))
