@@ -6,13 +6,13 @@ use std::cell::RefCell;
 use std::collections::HashSet;
 use std::sync::Arc;
 
+use im_ternary_tree::TernaryTreeList;
+
 use crate::builtins;
 use crate::call_stack::CallStackList;
 use crate::primes::{gen_core_id, Calcit, CalcitErr, CalcitItems, CalcitScope};
 use crate::runner;
 
-use im_ternary_tree::TernaryTreeList;
-
 pub fn defn(expr: &CalcitItems, scope: &CalcitScope, file_ns: Arc<str>) -> Result<Calcit, CalcitErr> {
   match (expr.get(0), expr.get(1)) {
     (Some(Calcit::Symbol { sym: s, .. }), Some(Calcit::List(xs))) => Ok(Calcit::Fn {
@@ -20,7 +20,7 @@ pub fn defn(expr: &CalcitItems, scope: &CalcitScope, file_ns: Arc<str>) -> Resul
       def_ns: file_ns.to_owned(),
       id: gen_core_id(),
       scope: Arc::new(scope.to_owned()),
-      args: get_raw_args(xs)?,
+      args: Arc::new(get_raw_args(xs)?),
       body: Arc::new(expr.skip(2)?),
     }),
     (Some(a), Some(b)) => CalcitErr::err_str(format!("invalid args type for defn: {} , {}", a, b)),
@@ -34,7 +34,7 @@ pub fn defmacro(expr: &CalcitItems, _scope: &CalcitScope, def_ns: Arc<str>) -> R
       name: s.to_owned(),
       def_ns: def_ns.to_owned(),
       id: gen_core_id(),
-      args: get_raw_args(xs)?,
+      args: Arc::new(get_raw_args(xs)?),
       body: Arc::new(expr.skip(2)?),
     }),
     (Some(a), Some(b)) => CalcitErr::err_str(format!("invalid structure for defmacro: {} {}", a, b)),
@@ -42,7 +42,7 @@ pub fn defmacro(expr: &CalcitItems, _scope: &CalcitScope, def_ns: Arc<str>) -> R
   }
 }
 
-pub fn get_raw_args(args: &CalcitItems) -> Result<Arc<Vec<Arc<str>>>, String> {
+pub fn get_raw_args(args: &CalcitItems) -> Result<Vec<Arc<str>>, String> {
   let mut xs: Vec<Arc<str>> = vec![];
   for item in args {
     if let Calcit::Symbol { sym, .. } = item {
@@ -51,7 +51,7 @@ pub fn get_raw_args(args: &CalcitItems) -> Result<Arc<Vec<Arc<str>>>, String> {
       return Err(format!("Unexpected argument: {}", item));
     }
   }
-  Ok(Arc::new(xs))
+  Ok(xs)
 }
 
 pub fn quote(expr: &CalcitItems, _scope: &CalcitScope, _file_ns: Arc<str>) -> Result<Calcit, CalcitErr> {
@@ -91,7 +91,7 @@ pub fn eval(expr: &CalcitItems, scope: &CalcitScope, file_ns: Arc<str>, call_sta
 
 pub fn syntax_let(expr: &CalcitItems, scope: &CalcitScope, file_ns: Arc<str>, call_stack: &CallStackList) -> Result<Calcit, CalcitErr> {
   match expr.get(0) {
-    Some(Calcit::Nil) => runner::evaluate_lines(&expr.skip(1)?, scope, file_ns.to_owned(), call_stack),
+    Some(Calcit::Nil) => runner::evaluate_lines(&expr.drop_left(), scope, file_ns.to_owned(), call_stack),
     Some(Calcit::List(xs)) if xs.len() == 2 => {
       let mut body_scope = scope.to_owned();
       match (&xs[0], &xs[1]) {
@@ -101,7 +101,7 @@ pub fn syntax_let(expr: &CalcitItems, scope: &CalcitScope, file_ns: Arc<str>, ca
         }
         (a, _) => return CalcitErr::err_str(format!("invalid binding name: {}", a)),
       }
-      runner::evaluate_lines(&expr.skip(1)?, &body_scope, file_ns.to_owned(), call_stack)
+      runner::evaluate_lines(&expr.drop_left(), &body_scope, file_ns.to_owned(), call_stack)
     }
     Some(Calcit::List(xs)) => CalcitErr::err_str(format!("invalid length: {:?}", xs)),
     Some(_) => CalcitErr::err_str(format!("invalid node for &let: {:?}", expr)),
@@ -113,7 +113,7 @@ pub fn syntax_let(expr: &CalcitItems, scope: &CalcitScope, file_ns: Arc<str>, ca
 #[derive(Clone, PartialEq, Debug)]
 enum SpanResult {
   Single(Calcit),
-  Range(CalcitItems),
+  Range(Box<CalcitItems>),
 }
 
 pub fn quasiquote(expr: &CalcitItems, scope: &CalcitScope, file_ns: Arc<str>, call_stack: &CallStackList) -> Result<Calcit, CalcitErr> {
@@ -144,18 +144,18 @@ fn replace_code(c: &Calcit, scope: &CalcitScope, file_ns: Arc<str>, call_stack:
       (Some(Calcit::Symbol { sym, .. }), Some(expr)) if &**sym == "~@" => {
         let ret = runner::evaluate_expr(expr, scope, file_ns.to_owned(), call_stack)?;
         match ret {
-          Calcit::List(zs) => Ok(SpanResult::Range(zs)),
+          Calcit::List(zs) => Ok(SpanResult::Range(Box::new(zs))),
           _ => Err(CalcitErr::use_str(format!("unknown result from unquote-slice: {}", ret))),
         }
       }
       (_, _) => {
-        let mut ret = TernaryTreeList::Empty;
+        let mut ret: TernaryTreeList<Calcit> = TernaryTreeList::Empty;
         for y in ys {
           match replace_code(y, scope, file_ns.to_owned(), call_stack)? {
-            SpanResult::Single(z) => ret = ret.push(z),
+            SpanResult::Single(z) => ret = ret.push_right(z),
             SpanResult::Range(pieces) => {
-              for piece in &pieces {
-                ret = ret.push(piece.to_owned());
+              for piece in &*pieces {
+                ret = ret.push_right(piece.to_owned());
               }
             }
           }
@@ -191,7 +191,7 @@ pub fn macroexpand(
   if expr.len() == 1 {
     let quoted_code = runner::evaluate_expr(&expr[0], scope, file_ns.to_owned(), call_stack)?;
 
-    match quoted_code.to_owned() {
+    match &quoted_code {
       Calcit::List(xs) => {
         if xs.is_empty() {
           return Ok(quoted_code);
@@ -200,15 +200,15 @@ pub fn macroexpand(
         match v {
           Calcit::Macro { def_ns, args, body, .. } => {
             // mutable operation
-            let mut rest_nodes = xs.skip(1)?;
+            let mut rest_nodes = xs.drop_left();
             // println!("macro: {:?} ... {:?}", args, rest_nodes);
             // keep expanding until return value is not a recur
             loop {
-              let body_scope = runner::bind_args(args.to_owned(), &rest_nodes, scope, call_stack)?;
+              let body_scope = runner::bind_args(&args, &rest_nodes, scope, call_stack)?;
               let v = runner::evaluate_lines(&body, &body_scope, def_ns.to_owned(), call_stack)?;
               match v {
                 Calcit::Recur(rest_code) => {
-                  rest_nodes = rest_code;
+                  rest_nodes = rest_code.to_owned();
                 }
                 _ => return Ok(v),
               }
@@ -217,7 +217,7 @@ pub fn macroexpand(
           _ => Ok(quoted_code),
         }
       }
-      a => Ok(a),
+      a => Ok(a.to_owned()),
     }
   } else {
     CalcitErr::err_str(format!("macroexpand expected excaclty 1 argument, got: {:?}", expr))
@@ -233,7 +233,7 @@ pub fn macroexpand_1(
   if expr.len() == 1 {
     let quoted_code = runner::evaluate_expr(&expr[0], scope, file_ns.to_owned(), call_stack)?;
     // println!("quoted: {}", quoted_code);
-    match quoted_code.to_owned() {
+    match &quoted_code {
       Calcit::List(xs) => {
         if xs.is_empty() {
           return Ok(quoted_code);
@@ -241,13 +241,13 @@ pub fn macroexpand_1(
         let v = runner::evaluate_expr(&xs[0], scope, file_ns.to_owned(), call_stack)?;
         match v {
           Calcit::Macro { def_ns, args, body, .. } => {
-            let body_scope = runner::bind_args(args.to_owned(), &xs.skip(1)?, scope, call_stack)?;
+            let body_scope = runner::bind_args(&args, &xs.drop_left(), scope, call_stack)?;
             runner::evaluate_lines(&body, &body_scope, def_ns, call_stack)
           }
           _ => Ok(quoted_code),
         }
       }
-      a => Ok(a),
+      a => Ok(a.to_owned()),
     }
   } else {
     CalcitErr::err_str(format!("macroexpand expected excaclty 1 argument, got: {:?}", expr))
@@ -263,7 +263,7 @@ pub fn macroexpand_all(
   if expr.len() == 1 {
     let quoted_code = runner::evaluate_expr(&expr[0], scope, file_ns.to_owned(), call_stack)?;
 
-    match quoted_code.to_owned() {
+    match &quoted_code {
       Calcit::List(xs) => {
         if xs.is_empty() {
           return Ok(quoted_code);
@@ -272,16 +272,16 @@ pub fn macroexpand_all(
         match v {
           Calcit::Macro { def_ns, args, body, .. } => {
             // mutable operation
-            let mut rest_nodes = xs.skip(1)?;
+            let mut rest_nodes = xs.drop_left();
             let check_warnings: &RefCell<Vec<String>> = &RefCell::new(vec![]);
             // println!("macro: {:?} ... {:?}", args, rest_nodes);
             // keep expanding until return value is not a recur
             loop {
-              let body_scope = runner::bind_args(args.to_owned(), &rest_nodes, scope, call_stack)?;
+              let body_scope = runner::bind_args(&args, &rest_nodes, scope, call_stack)?;
               let v = runner::evaluate_lines(&body, &body_scope, def_ns.to_owned(), call_stack)?;
               match v {
                 Calcit::Recur(rest_code) => {
-                  rest_nodes = rest_code;
+                  rest_nodes = rest_code.to_owned();
                 }
                 _ => {
                   let (resolved, _v) =
@@ -312,7 +312,7 @@ pub fn macroexpand_all(
           }
         }
       }
-      a => Ok(a),
+      a => Ok(a.to_owned()),
     }
   } else {
     CalcitErr::err_str(format!("macroexpand expected excaclty 1 argument, got: {:?}", expr))
@@ -334,7 +334,7 @@ pub fn call_try(expr: &CalcitItems, scope: &CalcitScope, file_ns: Arc<str>, call
             def_ns, scope, args, body, ..
           } => {
             let values = TernaryTreeList::from(&[err_data]);
-            runner::run_fn(&values, &scope, args.to_owned(), &body, def_ns, call_stack)
+            runner::run_fn(&values, &scope, &args, &body, def_ns, call_stack)
           }
           Calcit::Proc(proc) => builtins::handle_proc(&proc, &TernaryTreeList::from(&[err_data]), call_stack),
           a => CalcitErr::err_str(format!("try expected a function handler, got: {}", a)),
