@@ -2,6 +2,8 @@ pub mod gen_stack;
 mod internal_states;
 mod snippets;
 
+use im_ternary_tree::TernaryTreeList;
+
 use std::cell::RefCell;
 use std::collections::{BTreeMap, HashMap, HashSet};
 use std::fs;
@@ -9,7 +11,6 @@ use std::path::Path;
 use std::sync::Arc;
 
 use cirru_edn::EdnKwd;
-use im_ternary_tree::TernaryTreeList;
 
 use crate::builtins::meta::{js_gensym, reset_js_gensym_index};
 use crate::builtins::syntax::get_raw_args;
@@ -285,7 +286,7 @@ fn gen_call_code(
   }
 
   let head = ys[0].to_owned();
-  let body = ys.skip(1)?;
+  let body = ys.drop_left();
   match &head {
     Calcit::Syntax(s, ..) => {
       match s {
@@ -343,7 +344,7 @@ fn gen_call_code(
           (Some(Calcit::Symbol { sym, .. }), Some(Calcit::List(ys))) => {
             let func_body = body.skip(2)?;
             gen_stack::push_call_stack(ns, sym, StackKind::Codegen, xs.to_owned(), &TernaryTreeList::Empty);
-            let ret = gen_js_func(sym, get_raw_args(ys)?, &func_body, ns, false, local_defs, file_imports, keywords);
+            let ret = gen_js_func(sym, &get_raw_args(ys)?, &func_body, ns, false, local_defs, file_imports, keywords);
             gen_stack::pop_call_stack();
             match ret {
               Ok(code) => Ok(format!("{}{}", return_code, code)),
@@ -416,7 +417,7 @@ fn gen_call_code(
 
         "echo" | "println" => {
           // not core syntax, but treat as macro for better debugging experience
-          let args = ys.skip(1)?;
+          let args = ys.drop_left();
           let args_code = gen_args_code(&args, ns, local_defs, file_imports, keywords)?;
           Ok(format!("console.log({}printable({}))", proc_prefix, args_code))
         }
@@ -433,7 +434,7 @@ fn gen_call_code(
         }
         "new" => match body.get(0) {
           Some(ctor) => {
-            let args = body.skip(1)?;
+            let args = body.drop_left();
             let args_code = gen_args_code(&args, ns, local_defs, file_imports, keywords)?;
             Ok(format!(
               "{}new {}({})",
@@ -511,7 +512,7 @@ fn gen_call_code(
           if matches_js_var(name) {
             match body.get(0) {
               Some(obj) => {
-                let args = body.skip(1)?;
+                let args = body.drop_left();
                 let args_code = gen_args_code(&args, ns, local_defs, file_imports, keywords)?;
                 Ok(format!(
                   "{}{}.{}({})",
@@ -531,7 +532,7 @@ fn gen_call_code(
           let name = s.strip_prefix('.').unwrap();
           match body.get(0) {
             Some(obj) => {
-              let args = body.skip(1)?;
+              let args = body.drop_left();
               let args_code = gen_args_code(&args, ns, local_defs, file_imports, keywords)?;
               Ok(format!(
                 "{}{}invoke_method({})({},{})",
@@ -696,7 +697,7 @@ fn gen_let_code(
       return Err(format!("&let expected body, but got empty, {}", xs.lisp_str()));
     }
     let pair = let_def_body[0].to_owned();
-    let content = let_def_body.skip(1)?;
+    let content = let_def_body.drop_left();
 
     match &pair {
       Calcit::Nil => {
@@ -754,7 +755,7 @@ fn gen_let_code(
                   Calcit::List(ys) if ys.len() > 2 => match (&ys[0], &ys[1]) {
                     (Calcit::Syntax(sym, _ns), Calcit::List(zs)) if sym == &CalcitSyntax::CoreLet && zs.len() == 2 => match &zs[0] {
                       Calcit::Symbol { sym: s2, .. } if !scoped_defs.contains(s2) => {
-                        let_def_body = ys.skip(1)?;
+                        let_def_body = ys.drop_left();
                         continue;
                       }
                       _ => (),
@@ -940,7 +941,7 @@ fn uses_recur(xs: &Calcit) -> bool {
 
 fn gen_js_func(
   name: &str,
-  args: Arc<Vec<Arc<str>>>,
+  args: &[Arc<str>],
   raw_body: &CalcitItems,
   ns: &str,
   exported: bool,
@@ -1015,7 +1016,7 @@ fn gen_js_func(
         }
       }
     }
-    body = body.push(line.to_owned());
+    body = body.push_right(line.to_owned());
   }
 
   if !body.is_empty() && uses_recur(&body[body.len() - 1]) {
@@ -1239,16 +1240,7 @@ pub fn emit_js(entry_ns: &str, emit_path: &str) -> Result<(), String> {
           ..
         } => {
           gen_stack::push_call_stack(def_ns, name, StackKind::Codegen, f.to_owned(), &TernaryTreeList::Empty);
-          defs_code.push_str(&gen_js_func(
-            &def,
-            args.to_owned(),
-            code,
-            &ns,
-            true,
-            &def_names,
-            &file_imports,
-            &keywords,
-          )?);
+          defs_code.push_str(&gen_js_func(&def, args, code, &ns, true, &def_names, &file_imports, &keywords)?);
           gen_stack::pop_call_stack();
         }
         Calcit::Thunk(code, _) => {
