@@ -26,7 +26,7 @@ pub fn load_core_snapshot() -> Result<snapshot::Snapshot, String> {
   let bytes = include_bytes!("./cirru/calcit-core.cirru");
   let core_content = String::from_utf8_lossy(bytes).to_string();
   let core_data = cirru_edn::parse(&core_content)?;
-  snapshot::load_snapshot_data(core_data, "calcit-internal://calcit-core.cirru")
+  snapshot::load_snapshot_data(&core_data, "calcit-internal://calcit-core.cirru")
 }
 
 #[derive(Clone, Debug)]
@@ -73,7 +73,7 @@ pub fn run_program(init_ns: Arc<str>, init_def: Arc<str>, params: CalcitItems) -
       Calcit::Fn {
         def_ns, scope, args, body, ..
       } => {
-        let result = runner::run_fn(&params, &scope, args.to_owned(), &body, def_ns, &rpds::List::new_sync());
+        let result = runner::run_fn(&params, &scope, &args, &body, def_ns, &rpds::List::new_sync());
         match result {
           Ok(v) => Ok(v),
           Err(failure) => {
@@ -115,6 +115,6 @@ pub fn load_module(path: &str, base_dir: &Path) -> Result<snapshot::Snapshot, St
   let content = fs::read_to_string(&fullpath).unwrap_or_else(|_| panic!("expected Cirru snapshot {:?}", fullpath));
   let data = cirru_edn::parse(&content)?;
   // println!("reading: {}", content);
-  let snapshot = snapshot::load_snapshot_data(data, &fullpath)?;
+  let snapshot = snapshot::load_snapshot_data(&data, &fullpath)?;
   Ok(snapshot)
 }
